/*

    MiniMono Example 3 for Linux

    (C) Eugene Karataev

    http://www.minimdb.com
    support@minimdb.com


    Example demonstrates how to operate with $list structures
    and how to decorate strings to a MUMPS expression syntax.

*/

#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>  
#include <unistd.h>
#include <dlfcn.h>
#include <signal.h>
#include <strings.h>
#define stricmp strcasecmp

#include "minimono.h"

// virtual machine settings
MINIMONOVM vm;

int main(int argc, char* argv[])
{
  printf( "MiniMono example 3\n");

  freeminimonovm_t        FreeMiniMonoVM;
  createminimonovm_t      CreateMiniMonoVM;
  getdefaultsettingsvm_t  GetDefaultSettingsVM;
  mnmlistlength_t         MNMListLength;
  mnmlistget_t            MNMListGet;
  mnmlistset_t            MNMListSet;
  mnmtext_t               MNMText;

  int ret;
  MINIM_STR command;
  MINIM_STR expr;
  MINIM_STR list;
  MINIM_STR value;
  MINIM_STR str;
  int n;
  int i;

  // dynamic load MiniMono DLL
  void* hModule = dlopen( "./libminimono.so", RTLD_LAZY);
  if( !hModule)
  {
    printf( "dlopen failed\n");
    goto exit;
  }

  // import functions by name

  CreateMiniMonoVM =
    (createminimonovm_t)dlsym( hModule, "CreateMiniMonoVM");
  if( !CreateMiniMonoVM)
  {
    printf( "Function CreateMiniMonoVM not found\n");
    goto exit;
  }

  FreeMiniMonoVM =
    (freeminimonovm_t)dlsym( hModule, "FreeMiniMonoVM");
  if( !FreeMiniMonoVM)
  {
    printf( "Function FreeMiniMonoVM not found\n");
    goto exit;
  }

  GetDefaultSettingsVM =
    (getdefaultsettingsvm_t)dlsym( hModule, "GetDefaultSettingsVM");
  if( !GetDefaultSettingsVM)
  {
    printf( "Function GetDefaultSettingsVM not found\n");
    goto exit;
  }

  MNMListLength = (mnmlistlength_t)dlsym( hModule, "MNMListLength");
  if( !MNMListLength)
  {
    printf( "Function MNMListLength not found\n");
    goto exit;
  }

  MNMListGet = (mnmlistget_t)dlsym( hModule, "MNMListGet");
  if( !MNMListGet)
  {
    printf( "Function MNMListGet not found\n");
    goto exit;
  }

  MNMListSet = (mnmlistset_t)dlsym( hModule, "MNMListSet");
  if( !MNMListSet)
  {
    printf( "Function MNMListSet not found\n");
    goto exit;
  }

  MNMText = (mnmtext_t)dlsym( hModule, "MNMText");
  if( !MNMText)
  {
    printf( "Function MNMText not found\n");
    goto exit;
  }

  // get default MiniMono settings
  GetDefaultSettingsVM( &vm);

  // assign custom settings for this application
  vm.DataFile = "empty.dat";    // assign datafile name you are using
  vm.DBCacheSize = 100;         // 100 Mbytes
  vm.JournalingEnabled = 0;     // this example does not require journaling

  ret = CreateMiniMonoVM( &vm);
  switch( ret)
  {
    case MINIMONO_SUCCESS:
      printf( "CreateMiniMonoVM: MiniMono virtual machine created\n");
      break;
    case MINIMONO_CREATED:
      printf( "CreateMiniMonoVM: MiniMono virtual machine already been created\n");
      goto exit;
    default:
      printf( "Function CreateMiniMonoVM failed\n");
      goto exit;
  }

  // call virtual machine to create $list structure
  command.len = sprintf( (char*)command.data, "s var=$lb($zv,$sy,$zd($h,8))");
  vm.cbfunc->Execute( &command);

  // read value of an expression
  expr.len = sprintf( (char*)expr.data, "var");
  vm.cbfunc->Eval( &expr, &list);

  // display list length was read de facto
  n = MNMListLength( &list);
  printf( "Actual length of the list is: %d\n", n);

  // unpack and display list items separately
  for( int i = 1; i <= n; i ++)
  {
    // unpack i item from list into value
    MNMListGet( &list, i, &value);
    // convert into string in any case
    vm.cbfunc->GetStr( &value, &str);
    // show value as a string
    printf( "Item %d : %.*s\n", i, str.len, str.data);
  }

  // create list structure
  // initial value of the list is an empty string
  list.len = 0;

  // add integer value
  vm.cbfunc->SetInt32( 123456, &value);
  // set as 1 list item
  MNMListSet( &list, 1, &value);

  // add double value
  vm.cbfunc->SetDouble( 123.456, &value);
  // set as 2 list item
  MNMListSet( &list, 2, &value);

  // add string value
  value.len = sprintf( (char*)value.data, "Hello, lists!");
  // set as 3 list item
  MNMListSet( &list, 3, &value);

  // write to local variable
  vm.cbfunc->WriteLocal( "varname", 0, NULL, &list);

  // read list items from MUMPS context
  for( int i = 1; i <= 3; i++)
  {
    expr.len = sprintf( (char*)expr.data, "$lg(varname,%d)", i);
    vm.cbfunc->Eval( &expr, &value);
    // values can have non-string format, convert before display
    vm.cbfunc->GetStr( &value, &str);
    printf( "List item %d : %.*s\n", i, str.len, str.data);
  }

  // show special symbols decoration
  printf( "Strings decorated with MUMPS expression syntax:\n");

  // number
  value.len = sprintf( (char*)value.data, "%G", 123.456);
  MNMText( &value, &str);
  // result is always a string, separate conversion does not required
  printf( "Result: %.*s\n", str.len, str.data);

  // printable string
  value.len = sprintf( (char*)value.data, "%s", "Hello, world!");
  MNMText( &value, &str);
  printf( "Result: %.*s\n", str.len, str.data);

  // string with nonprintable characters
  value.len = sprintf( (char*)value.data, "%s",
    "String\r\nwith line feeds\r\n\tand tabs\t");
  MNMText( &value, &str);
  printf( "Result: %.*s\n", str.len, str.data);

  // free virtual machine
  FreeMiniMonoVM();

exit:

  if( hModule)
  {
    dlclose( hModule);
  }

  printf( "Example finished, press any key...\n");
  getchar();
  return 0;
};

