/*

    MiniMono Example 5 for Linux

    (C) Eugene Karataev

    http://www.minimdb.com
    support@minimdb.com


    Example demonstrates how to directly call globals

*/

#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>  
#include <unistd.h>
#include <dlfcn.h>
#include <signal.h>
#include <strings.h>
#define stricmp strcasecmp

#include "minimono.h"

// virtual machine settings
MINIMONOVM vm;

static char* database_name = "%SYS";
static char* global_name = "ABC";

int main(int argc, char* argv[])
{
  printf( "MiniMono example 5\n");

  freeminimonovm_t        FreeMiniMonoVM;
  createminimonovm_t      CreateMiniMonoVM;
  getdefaultsettingsvm_t  GetDefaultSettingsVM;
  int ret;
  const int index_count = 2;
  MINIM_STR index_value[ index_count];
  MINIM_STR* index[ index_count];
  MINIM_STR value;
  MINIM_STR str;

  // dynamic load MiniMono DLL
  void* hModule = dlopen( "./libminimono.so", RTLD_LAZY);
  if( !hModule)
  {
    printf( "dlopen failed\n");
    goto exit;
  }

  // import functions by name
  CreateMiniMonoVM =
    (createminimonovm_t)dlsym( hModule, "CreateMiniMonoVM");
  if( !CreateMiniMonoVM)
  {
    printf( "Function CreateMiniMonoVM not found\n");
    goto exit;
  }

  FreeMiniMonoVM =
    (freeminimonovm_t)dlsym( hModule, "FreeMiniMonoVM");
  if( !FreeMiniMonoVM)
  {
    printf( "Function FreeMiniMonoVM not found\n");
    goto exit;
  }

  GetDefaultSettingsVM =
    (getdefaultsettingsvm_t)dlsym( hModule, "GetDefaultSettingsVM");
  if( !GetDefaultSettingsVM)
  {
    printf( "Function GetDefaultSettingsVM not found\n");
    goto exit;
  }

  // get default MiniMono settings
  GetDefaultSettingsVM( &vm);

  // assign custom settings for this application
  vm.DataFile = "empty.dat";    // assign datafile name you are using
  vm.DBCacheSize = 100;         // 100 Mbytes
  vm.JournalingEnabled = 0;     // this example does not require journaling

  ret = CreateMiniMonoVM( &vm);
  switch( ret)
  {
    case MINIMONO_SUCCESS:
      printf( "CreateMiniMonoVM: MiniMono virtual machine created\n");
      break;
    case MINIMONO_CREATED:
      printf( "CreateMiniMonoVM: MiniMono virtual machine already been created\n");
      goto exit;
    default:
      printf( "Function CreateMiniMonoVM failed\n");
      goto exit;
  }

  // create array of index as array of pointers
  index[ 0] = &index_value[ 0];
  index[ 1] = &index_value[ 1];

  // write global values
  for( int i = 0; i < 8; i++)
  {
    // create index values
    vm.cbfunc->SetInt32( i, index[ 0]);
    vm.cbfunc->SetInt32( i * i, index[ 1]);
    // create value
    vm.cbfunc->SetInt32( i * i * i, &value);
    // write value to global
    vm.cbfunc->WriteGlobal( global_name, database_name,
      index_count, index, &value);
  }

  // read global values
  for( int i = 0; i < 8; i++)
  {
    // create index values
    vm.cbfunc->SetInt32( i, index[ 0]);
    vm.cbfunc->SetInt32( i * i, index[ 1]);
    // read value from global
    ret = vm.cbfunc->ReadGlobal( global_name, database_name,
      index_count, index, &value);
    printf( "Value of ^%s(%d,%d) : ", global_name, i, i * i);
    switch( ret)
    {
      case ZDLL_CALLBACK_DONE:
        // convert to string
        vm.cbfunc->GetStr( &value, &str);
        printf( "%.*s\n", str.len, str.data);
        break;
      case ZDLL_CALLBACK_UNDEFINED:
        printf( "<UNDEFINED>\n");
        break;
      default:
        printf( "UNEXPECTED ERROR\n");
        break;
    }
  }

  // kill some globals
  for( int i = 0; i < 8; i += 2)
  {
    // create index values
    vm.cbfunc->SetInt32( i, index[ 0]);
    vm.cbfunc->SetInt32( i * i, index[ 1]);
    // kill global
    vm.cbfunc->KillGlobal( global_name, database_name, index_count, index);
  }

  // read global values again
  for( int i = 0; i < 8; i++)
  {
    // create index values
    vm.cbfunc->SetInt32( i, index[ 0]);
    vm.cbfunc->SetInt32( i * i, index[ 1]);
    // read value from global
    ret = vm.cbfunc->ReadGlobal( global_name, database_name, 2, index, &value);
    printf( "Value of ^%s(%d,%d) : ", global_name, i, i * i);
    switch( ret)
    {
      case ZDLL_CALLBACK_DONE:
        // convert to string
        vm.cbfunc->GetStr( &value, &str);
        printf( "%.*s\n", str.len, str.data);
        break;
      case ZDLL_CALLBACK_UNDEFINED:
        printf( "<UNDEFINED>\n");
        break;
      default:
        printf( "UNEXPECTED ERROR\n");
        break;
    }
  }

  // free virtual machine
  FreeMiniMonoVM();

exit:

  if( hModule)
  {
    dlclose( hModule);
  }

  printf( "Example finished, press any key...\n");
  getchar();
  return 0;
};
 
