/*

    MiniM

    External ZDevice example for Linux

    Example is read-only device to return fixed symbol's sequence.
    Sequence is randomized while device is opened.

*/

#ifdef _WIN32
#include <windows.h>
#endif

#include <memory.h>
#include <malloc.h>
#include <stdlib.h>
#include <time.h>

#ifndef _WIN32
#include <ctype.h>
#endif

#include "zdevice.h"

static const int sequence_len = 1024 * 128;

// internal device data 
struct ZDev1Data
{
  int position;
  char data[ sequence_len];
};

#ifndef _WIN32
int memicmp( const void* p1, const void* p2, size_t n)
{
  const char* ch1 = (const char*)p1;
  const char* ch2 = (const char*)p2;
  for( ; n; ch1++, ch2++, n--)
  {
    int res = toupper( *ch1) - toupper( *ch2);
    if( res)
      return res;
  }
  return 0;
};
#endif

// on open we initiate some data
// dev_name parameter is ignored
// timeout and options are ignored
ZDEVICEHANDLE STDCALL DevOpen( ZDLLCB* cbfunc, const char* dev_name,
  MINIM_STR** param_pairs, int param_pairs_count, int timeout)
{
  ZDev1Data* ret = (ZDev1Data*)malloc( sizeof( ZDev1Data));
  ret->position = 0;
  srand( (unsigned)time( NULL));
  for( int i = 0; i < sequence_len; i++)
  {
    ret->data[ i] = 'A' + (int)(((double) rand() / 
                         (double) RAND_MAX) * 26);
  }
  return (ZDEVICEHANDLE)ret;
};

// handle device closing
int STDCALL DevClose( ZDLLCB* cbfunc, ZDEVICEHANDLE hDevice,
  MINIM_STR** param_pairs, int param_pairs_count)
{
  ZDev1Data* data = (ZDev1Data*)hDevice;
  free( data);
  return ZDEV_NO_ERROR;
};

// handle string read
int STDCALL DevReadStr( ZDLLCB* cbfunc, ZDEVICEHANDLE hDevice,
  int len, int timeout, MINIM_STR* result)
{
  ZDev1Data* data = (ZDev1Data*)hDevice;
  if( data->position >= sequence_len - 1)
  {
    // nothing to read now
    return ZDEV_ERR_ENDOFFILE;
  }
  // maximum MiniM string length
  int max_read = 32 * 1024 - 1;
  // check default read length
  int len_to_read = ( (len < 0) || (len > max_read) ) ? max_read : len;
  // check read limit
  if( len_to_read > sequence_len - data->position)
  {
    len_to_read = sequence_len - data->position;
  }
  memcpy( result->data, data->data + data->position, len_to_read);
  result->len = len_to_read;
  data->position += len_to_read;
  return ZDEV_NO_ERROR;
};

// handle $zeof query
int STDCALL DevZEOF( ZDLLCB* cbfunc, ZDEVICEHANDLE hDevice, int* result)
{
  // return end-of-file indicator
  ZDev1Data* data = (ZDev1Data*)hDevice;
  if( data->position >= sequence_len - 1 )
  {
    // yes, we are at the end of the sequence
    *result = 1;
  }
  else
  {
    // no, end is not reached
    *result = 0;
  }
  return ZDEV_NO_ERROR;
};

// handle char read
int STDCALL DevReadChar( ZDLLCB* cbfunc, ZDEVICEHANDLE hDevice,
  int timeout, int* result)
{
  ZDev1Data* data = (ZDev1Data*)hDevice;
  if( data->position >= sequence_len - 1)
  {
    // nothing to read now
    return ZDEV_ERR_ENDOFFILE;
  }
  *result = (unsigned)data->data[ data->position];
  data->position ++;
  return ZDEV_NO_ERROR;
};

// handle use command and apply options
int STDCALL DevUse( ZDLLCB* cbfunc, ZDEVICEHANDLE hDevice,
  MINIM_STR** param_pairs, int param_pairs_count)
{
  ZDev1Data* data = (ZDev1Data*)hDevice;
  // here we search option /POS=value inside options
  for( int i = 0; i < param_pairs_count; i++)
  {
    MINIM_STR* name = param_pairs[ i * 2];
    MINIM_STR* value = param_pairs[ i * 2 + 1];
    // need case when both name and value are defined
    if( name && value)
    {
      // name need to be POS
      if( ( name->len == 3 ) && ( memicmp( name->data, "POS", 3) == 0 ) )
      {
        // ok, POS option is found
        // convert value to integer
        int ivalue = cbfunc->GetInt32( value);
        // check position is outside of available data
        if( ivalue < 0)
        {
          ivalue = 0;
        }
        if( ivalue > sequence_len - 1)
        {
          ivalue = sequence_len - 1;
        }
        data->position = ivalue;
        break;
      }
    }
  }
  return ZDEV_NO_ERROR;
};

ZDEVICE zdev1;

// in Linux gcc exports all functions by default
#ifdef __cplusplus
extern "C" 
#endif
#ifdef _WIN32
__declspec(dllexport)
#endif
ZDEVICE* STDCALL GetZDevice( void)
{
  // zero pointers to unimplemented functions
  memset( &zdev1, 0, sizeof( zdev1));
  zdev1.Open = DevOpen;
  zdev1.Close = DevClose;
  zdev1.ReadStr = DevReadStr;
  zdev1.ReadChar = DevReadChar;
  zdev1.Use = DevUse;
  zdev1.GetZEOF = DevZEOF;
  return &zdev1;
};

#ifdef _WIN32
#pragma argsused
int WINAPI DllEntryPoint(HINSTANCE hinst, unsigned long reason, void* lpReserved)
{
  return 1;
};
#endif
